#include <stdlib.h>
#include <error.h>
#include <errno.h>
#include <unistd.h>
#include <pwd.h>
#include <grp.h>

#include "droppriv.h"

void
drop_priv (const char user[], const char dir[])
{
	struct passwd *pw;

	if (geteuid ())
		return;

	if (!(user && *user))
		error (EXIT_FAILURE, 0, "user shouldn't be empty");
	if (!(dir && *dir))
		error (EXIT_FAILURE, 0, "chroot directory shouldn't be empty");
	
	if ((setgroups (0, NULL) < 0))
		error (EXIT_FAILURE, errno, "setgroups");

	pw = getpwnam (user);
	if (!pw)
		error (EXIT_FAILURE, 0, "lookup of user \"%s\" failed", user);

	endpwent ();

	if (!pw->pw_uid)
		error (EXIT_FAILURE, 0, "user \"%s\" shouldn't be root",
		       user);

	if (chroot (dir) < 0)
		error (EXIT_FAILURE, errno, "chroot to \"%s\" failed", dir);
	if (chdir ("/") < 0)
		error (EXIT_FAILURE, errno, "chroot to \"/\" failed");

	if (setgid (pw->pw_gid) < 0)
		error (EXIT_FAILURE, errno, "setgid");

	if (setuid (pw->pw_uid) < 0)
		error (EXIT_FAILURE, errno, "setuid");
}
